@@ -1,4 +1,4 @@
-// Game_Music_Emu 0.5.5. http://www.slack.net/~ant/
+// Game_Music_Emu https://bitbucket.org/mpyne/game-music-emu/
 
 /*
 Last validated with zexall 2006.11.14 2:19 PM
@@ -136,7 +136,7 @@ void Kss_Cpu::map_mem( unsigned addr, blargg_ulong size, void* write, void const
 #define R16( n, shift, offset )\
 	(*(uint16_t*) ((char*) r16_ - (offset >> (shift - 1)) + ((n) >> (shift - 1))))
 
-#define CASE5( a, b, c, d, e          ) case 0x##a:case 0x##b:case 0x##c:case 0x##d:case 0x##e
+#define CASE5( a, b, c, d, e          ) /*FALLTHRU*/ case 0x##a:case 0x##b:case 0x##c:case 0x##d:case 0x##e
 #define CASE6( a, b, c, d, e, f       ) CASE5( a, b, c, d, e       ): case 0x##f
 #define CASE7( a, b, c, d, e, f, g    ) CASE6( a, b, c, d, e, f    ): case 0x##g
 #define CASE8( a, b, c, d, e, f, g, h ) CASE7( a, b, c, d, e, f, g ): case 0x##h
@@ -162,11 +162,6 @@ static byte const ed_dd_timing [0x100] = {
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
 };
 
-// even on x86, using short and unsigned char was slower
-typedef int         fint16;
-typedef unsigned    fuint16;
-typedef unsigned    fuint8;
-
 bool Kss_Cpu::run( cpu_time_t end_time )
 {
 	set_end_time( end_time );
@@ -174,8 +169,6 @@ bool Kss_Cpu::run( cpu_time_t end_time )
 	this->state = &s;
 	bool warning = false;
 	
-	typedef BOOST::int8_t int8_t;
-	
 	union {
 		regs_t rg;
 		pairs_t rp;
@@ -185,10 +178,10 @@ bool Kss_Cpu::run( cpu_time_t end_time )
 	rg = this->r.b;
 	
 	cpu_time_t s_time = s.time;
-	fuint16 pc = r.pc;
-	fuint16 sp = r.sp;
-	fuint16 ix = r.ix; // TODO: keep in memory for direct access?
-	fuint16 iy = r.iy;
+	uint_fast32_t pc = r.pc;
+	uint_fast32_t sp = r.sp;
+	uint_fast32_t ix = r.ix; // TODO: keep in memory for direct access?
+	uint_fast32_t iy = r.iy;
 	int flags = r.b.flags;
 	
 	goto loop;
@@ -210,7 +203,7 @@ loop:
 	uint8_t const* instr = s.read [pc >> page_shift];
 #define GET_ADDR()  GET_LE16( instr )
 	
-	fuint8 opcode;
+	uint_fast8_t opcode;
 	
 	// TODO: eliminate this special case
 	#if BLARGG_NONPORTABLE
@@ -243,7 +236,7 @@ loop:
 		11,10,10, 4,17,11, 7,11,11, 6,10, 4,17, 8, 7,11, // F
 	};
 	
-	fuint16 data;
+	uint_fast16_t data;
 	data = base_timing [opcode];
 	if ( (s_time += data) >= 0 )
 		goto possibly_out_of_time;
@@ -299,7 +292,7 @@ possibly_out_of_time:
 		goto loop;
 	
 	case 0x3A:{// LD A,(addr)
-		fuint16 addr = GET_ADDR();
+		uint_fast16_t addr = GET_ADDR();
 		pc += 2;
 		rg.a = READ( addr );
 		goto loop;
@@ -315,7 +308,7 @@ possibly_out_of_time:
 // JR
 // TODO: more efficient way to handle negative branch that wraps PC around
 #define JR( cond ) {\
-	int offset = (BOOST::int8_t) data;\
+	int offset = (int8_t) data;\
 	pc++;\
 	if ( !(cond) )\
 		goto jr_not_taken;\
@@ -387,7 +380,7 @@ possibly_out_of_time:
 	
 	case 0xCD:{// CALL addr
 	call_taken:
-		fuint16 addr = pc + 2;
+		uint_fast16_t addr = pc + 2;
 		pc = GET_ADDR();
 		sp = uint16_t (sp - 2);
 		WRITE_WORD( sp, addr );
@@ -397,6 +390,7 @@ possibly_out_of_time:
 	case 0xFF: // RST
 		if ( pc > idle_addr )
 			goto hit_idle_addr;
+		// FALLTHRU
 	CASE7( C7, CF, D7, DF, E7, EF, F7 ):
 		data = pc;
 		pc = opcode & 0x38;
@@ -432,7 +426,7 @@ possibly_out_of_time:
 // ADC/ADD/SBC/SUB
 	case 0x96: // SUB (HL)
 	case 0x86: // ADD (HL)
-		flags &= ~C01;
+		flags &= ~C01; // FALLTHRU
 	case 0x9E: // SBC (HL)
 	case 0x8E: // ADC (HL)
 		data = READ( rp.hl );
@@ -440,7 +434,7 @@ possibly_out_of_time:
 	
 	case 0xD6: // SUB A,imm
 	case 0xC6: // ADD imm
-		flags &= ~C01;
+		flags &= ~C01; // FALLTHRU
 	case 0xDE: // SBC A,imm
 	case 0xCE: // ADC imm
 		pc++;
@@ -693,21 +687,21 @@ possibly_out_of_time:
 		goto loop;
 	
 	case 0x2A:{// LD HL,(addr)
-		fuint16 addr = GET_ADDR();
+		uint_fast16_t addr = GET_ADDR();
 		pc += 2;
 		rp.hl = READ_WORD( addr );
 		goto loop;
 	}
 	
 	case 0x32:{// LD (addr),A
-		fuint16 addr = GET_ADDR();
+		uint_fast16_t addr = GET_ADDR();
 		pc += 2;
 		WRITE( addr, rg.a );
 		goto loop;
 	}
 	
 	case 0x22:{// LD (addr),HL
-		fuint16 addr = GET_ADDR();
+		uint_fast16_t addr = GET_ADDR();
 		pc += 2;
 		WRITE_WORD( addr, rp.hl );
 		goto loop;
@@ -730,7 +724,7 @@ possibly_out_of_time:
 // Rotate
 	
 	case 0x07:{// RLCA
-		fuint16 temp = rg.a;
+		uint_fast16_t temp = rg.a;
 		temp = (temp << 1) | (temp >> 7);
 		flags = (flags & (S80 | Z40 | P04)) |
 				(temp & (F20 | F08 | C01));
@@ -739,7 +733,7 @@ possibly_out_of_time:
 	}
 	
 	case 0x0F:{// RRCA
-		fuint16 temp = rg.a;
+		uint_fast16_t temp = rg.a;
 		flags = (flags & (S80 | Z40 | P04)) |
 				(temp & C01);
 		temp = (temp << 7) | (temp >> 1);
@@ -753,12 +747,12 @@ possibly_out_of_time:
 		flags = (flags & (S80 | Z40 | P04)) |
 				(temp & (F20 | F08)) |
 				(temp >> 8);
-		rg.a = temp;
+		rg.a = (uint8_t)temp;
 		goto loop;
 	}
 	
 	case 0x1F:{// RRA
-		fuint16 temp = (flags << 7) | (rg.a >> 1);
+		uint_fast16_t temp = (flags << 7) | (rg.a >> 1);
 		flags = (flags & (S80 | Z40 | P04)) |
 				(temp & (F20 | F08)) |
 				(rg.a & C01);
@@ -768,7 +762,7 @@ possibly_out_of_time:
 	
 // Misc
 	case 0x2F:{// CPL
-		fuint16 temp = ~rg.a;
+		uint_fast16_t temp = ~rg.a;
 		flags = (flags & (S80 | Z40 | P04 | C01)) |
 				(temp & (F20 | F08)) |
 				(H10 | N02);
@@ -794,21 +788,21 @@ possibly_out_of_time:
 		goto loop;
 
 	case 0xE3:{// EX (SP),HL
-		fuint16 temp = READ_WORD( sp );
+		uint_fast16_t temp = READ_WORD( sp );
 		WRITE_WORD( sp, rp.hl );
 		rp.hl = temp;
 		goto loop;
 	}
 	
 	case 0xEB:{// EX DE,HL
-		fuint16 temp = rp.hl;
+		uint_fast16_t temp = rp.hl;
 		rp.hl = rp.de;
 		rp.de = temp;
 		goto loop;
 	}
 	
 	case 0xD9:{// EXX DE,HL
-		fuint16 temp = r.alt.w.bc;
+		uint_fast16_t temp = r.alt.w.bc;
 		r.alt.w.bc = rp.bc;
 		rp.bc = temp;
 		
@@ -841,6 +835,7 @@ possibly_out_of_time:
 	case 0xCB:
 		unsigned data2;
 		data2 = instr [1];
+		(void) data2; // TODO is this the same as data in all cases?
 		pc++;
 		switch ( data )
 		{
@@ -848,7 +843,7 @@ possibly_out_of_time:
 	// Rotate left
 		
 	#define RLC( read, write ) {\
-		fuint8 result = read;\
+		uint_fast8_t result = read;\
 		result = uint8_t (result << 1) | (result >> 7);\
 		flags = SZ28P( result ) | (result & C01);\
 		write;\
@@ -867,7 +862,7 @@ possibly_out_of_time:
 		}
 		
 	#define RL( read, write ) {\
-		fuint16 result = (read << 1) | (flags & C01);\
+		uint_fast16_t result = (read << 1) | (flags & C01);\
 		flags = SZ28PC( result );\
 		write;\
 		goto loop;\
@@ -885,7 +880,7 @@ possibly_out_of_time:
 		}
 		
 	#define SLA( read, add, write ) {\
-		fuint16 result = (read << 1) | add;\
+		uint_fast16_t result = (read << 1) | add;\
 		flags = SZ28PC( result );\
 		write;\
 		goto loop;\
@@ -916,7 +911,7 @@ possibly_out_of_time:
 	// Rotate right
 		
 	#define RRC( read, write ) {\
-		fuint8 result = read;\
+		uint_fast8_t result = read;\
 		flags = result & C01;\
 		result = uint8_t (result << 7) | (result >> 1);\
 		flags |= SZ28P( result );\
@@ -936,8 +931,8 @@ possibly_out_of_time:
 		}
 		
 	#define RR( read, write ) {\
-		fuint8 result = read;\
-		fuint8 temp = result & C01;\
+		uint_fast8_t result = read;\
+		uint_fast8_t temp = result & C01;\
 		result = uint8_t (flags << 7) | (result >> 1);\
 		flags = SZ28P( result ) | temp;\
 		write;\
@@ -956,7 +951,7 @@ possibly_out_of_time:
 		}
 		
 	#define SRA( read, write ) {\
-		fuint8 result = read;\
+		uint_fast8_t result = read;\
 		flags = result & C01;\
 		result = (result & 0x80) | (result >> 1);\
 		flags |= SZ28P( result );\
@@ -976,7 +971,7 @@ possibly_out_of_time:
 		}
 		
 	#define SRL( read, write ) {\
-		fuint8 result = read;\
+		uint_fast8_t result = read;\
 		flags = result & C01;\
 		result >>= 1;\
 		flags |= SZ28P( result );\
@@ -1120,7 +1115,7 @@ possibly_out_of_time:
 		case 0x43: // LD (ADDR),BC
 		case 0x53: // LD (ADDR),DE
 				temp = R16( data, 4, 0x43 );
-			fuint16 addr = GET_ADDR();
+			uint_fast16_t addr = GET_ADDR();
 			pc += 2;
 			WRITE_WORD( addr, temp );
 			goto loop;
@@ -1128,21 +1123,21 @@ possibly_out_of_time:
 		
 		case 0x4B: // LD BC,(ADDR)
 		case 0x5B:{// LD DE,(ADDR)
-			fuint16 addr = GET_ADDR();
+			uint_fast16_t addr = GET_ADDR();
 			pc += 2;
 			R16( data, 4, 0x4B ) = READ_WORD( addr );
 			goto loop;
 		}
 		
 		case 0x7B:{// LD SP,(ADDR)
-			fuint16 addr = GET_ADDR();
+			uint_fast16_t addr = GET_ADDR();
 			pc += 2;
 			sp = READ_WORD( addr );
 			goto loop;
 		}
 		
 		case 0x67:{// RRD
-			fuint8 temp = READ( rp.hl );
+			uint_fast8_t temp = READ( rp.hl );
 			WRITE( rp.hl, (rg.a << 4) | (temp >> 4) );
 			temp = (rg.a & 0xF0) | (temp & 0x0F);
 			flags = (flags & C01) | SZ28P( temp );
@@ -1151,7 +1146,7 @@ possibly_out_of_time:
 		}
 		
 		case 0x6F:{// RLD
-			fuint8 temp = READ( rp.hl );
+			uint_fast8_t temp = READ( rp.hl );
 			WRITE( rp.hl, (temp << 4) | (rg.a & 0x0F) );
 			temp = (rg.a & 0xF0) | (temp >> 4);
 			flags = (flags & C01) | SZ28P( temp );
@@ -1175,7 +1170,7 @@ possibly_out_of_time:
 		case 0xA1: // CPI
 		case 0xB1: // CPIR
 				inc = +1;
-			fuint16 addr = rp.hl;
+			uint_fast16_t addr = rp.hl;
 			rp.hl = addr + inc;
 			int temp = READ( addr );
 			
@@ -1208,7 +1203,7 @@ possibly_out_of_time:
 		case 0xA0: // LDI
 		case 0xB0: // LDIR
 				inc = +1;
-			fuint16 addr = rp.hl;
+			uint_fast16_t addr = rp.hl;
 			rp.hl = addr + inc;
 			int temp = READ( addr );
 			
@@ -1240,7 +1235,7 @@ possibly_out_of_time:
 		case 0xA3: // OUTI
 		case 0xB3: // OTIR
 				inc = +1;
-			fuint16 addr = rp.hl;
+			uint_fast16_t addr = rp.hl;
 			rp.hl = addr + inc;
 			int temp = READ( addr );
 			
@@ -1266,7 +1261,7 @@ possibly_out_of_time:
 		case 0xB2: // INIR
 				inc = +1;
 			
-			fuint16 addr = rp.hl;
+			uint_fast16_t addr = rp.hl;
 			rp.hl = addr + inc;
 			
 			int temp = IN( rp.bc );
@@ -1331,7 +1326,7 @@ possibly_out_of_time:
 
 //////////////////////////////////////// DD/FD prefix
 	{
-	fuint16 ixy;
+	uint_fast16_t ixy;
 	case 0xDD:
 		ixy = ix;
 		goto ix_prefix;
@@ -1353,7 +1348,7 @@ possibly_out_of_time:
 	
 		case 0x96: // SUB (IXY+disp)
 		case 0x86: // ADD (IXY+disp)
-			flags &= ~C01;
+			flags &= ~C01; // FALLTHRU
 		case 0x9E: // SBC (IXY+disp)
 		case 0x8E: // ADC (IXY+disp)
 			pc++;
@@ -1363,7 +1358,7 @@ possibly_out_of_time:
 		
 		case 0x94: // SUB HXY
 		case 0x84: // ADD HXY
-			flags &= ~C01;
+			flags &= ~C01; // FALLTHRU
 		case 0x9C: // SBC HXY
 		case 0x8C: // ADC HXY
 			opcode = data;
@@ -1372,7 +1367,7 @@ possibly_out_of_time:
 		
 		case 0x95: // SUB LXY
 		case 0x85: // ADD LXY
-			flags &= ~C01;
+			flags &= ~C01; // FALLTHRU
 		case 0x9D: // SBC LXY
 		case 0x8D: // ADC LXY
 			opcode = data;
@@ -1527,7 +1522,7 @@ possibly_out_of_time:
 			goto loop;
 	
 		case 0x22:{// LD (ADDR),IXY
-			fuint16 addr = GET_ADDR();
+			uint_fast16_t addr = GET_ADDR();
 			pc += 2;
 			WRITE_WORD( addr, ixy );
 			goto loop;
@@ -1539,7 +1534,7 @@ possibly_out_of_time:
 			goto set_ixy;
 		
 		case 0x2A:{// LD IXY,(addr)
-			fuint16 addr = GET_ADDR();
+			uint_fast16_t addr = GET_ADDR();
 			ixy = READ_WORD( addr );
 			pc += 2;
 			goto set_ixy;
@@ -1563,7 +1558,7 @@ possibly_out_of_time:
 			case 0x3E: goto srl_data_addr; // SRL (IXY)
 			
 			CASE8( 46, 4E, 56, 5E, 66, 6E, 76, 7E ):{// BIT b,(IXY+disp)
-				fuint8 temp = READ( data );
+				uint_fast8_t temp = READ( data );
 				int masked = temp & 1 << (data2 >> 3 & 7);
 				flags = (flags & C01) | H10 |
 						(masked & S80) |
@@ -1665,7 +1660,7 @@ possibly_out_of_time:
 			goto loop;
 		
 		case 0xE3:{// EX (SP),IXY
-			fuint16 temp = READ_WORD( sp );
+			uint_fast16_t temp = READ_WORD( sp );
 			WRITE_WORD( sp, ixy );
 			ixy = temp;
 			goto set_ixy;
